#!/bin/sh
# transferir_iconos.sh - Aplicacin para transferir iconos de escritorio a los
# clientes.
# El funcionamiento se basa en sincronizar los ".desktop" de $CARPETA_ORIGEN con
# el /etc/skel de los clientes (para que en el arranque de los usuarios se creen
# los iconos en el escritorio).
# Habr que marcar convenientemente el nombre de los ".desktop" para
# diferenciarlos del resto de .desktop que pudieran existir previamente.

TITULO="Iconos en el escritorio"
NOMBRE_CARPETA_ORIGEN=iconos_escritorio
CARPETA_ORIGEN=~/Desktop/$NOMBRE_CARPETA_ORIGEN
PREFIJO=BIBLIO

# Estamos en X?
if [ ! $DISPLAY ]; then
  : ${DIALOG=dialog}
else
  : ${DIALOG=Xdialog}
fi

NUM_CLIENTES=9

muestra_mensaje_inicial()
{
  $DIALOG --title "$TITULO" --center --ok-label "Aceptar" \
          --msgbox "\
Bienvenido a la aplicacin de $TITULO.
Con esta aplicacin puede enviar a los equipos de usuario un conjunto
de aplicaciones para que les aparezcan en el escritorio.
Para ello, dispone de una carpeta en su escritorio con el ttulo
\"$NOMBRE_CARPETA_ORIGEN\", donde deber poner todas las aplicaciones que
quiera enviar a los usuarios.
Una vez lo haya hecho, pulse en el botn \"Aceptar\" para llevar a cabo
la transferencia." 0 0 10000
  return 0
}

cuenta_ficheros_para_transmitir()
{
  NUM_FICHEROS_TRANSMITIR=`ls $CARPETA_ORIGEN/*.desktop | xargs -r -n 1 2> /dev/null | wc -l`
  return 0
}

prepara_nombres_origen()
{
  if [ "$NUM_FICHEROS_TRANSMITIR" -eq "0" ]; then
    return 1
  else 
    TMPDIR=`mktemp -d`
    for i in $CARPETA_ORIGEN/*.desktop
    do
    	cp $i $TMPDIR/${PREFIJO}_`basename $i`
    done
  fi
  return 0
}

envia_ficheros_clientes()
{
  if [ "$NUM_FICHEROS_TRANSMITIR" -eq "0" ]; then
    $DIALOG --title "$TITULO" --center --ok-label "Aceptar" \
          --msgbox "\
No hay ninguna aplicacin para transmitir a los clientes. Por favor, ponga alguna
aplicacin en la carpeta \"$NOMBRE_CARPETA_ORIGEN\" en el escritorio y vuelva a ejecutar esta
aplicacin." 0 0 10000
    return 1
  fi
  
(  i=1
  while [ $i -le $NUM_CLIENTES ]
  do
    # Se borran remotamente los fichero $PREFIJO_* y se reenvan los nuevos
    ssh cliente-$i rm /etc/skel/Desktop/${PREFIJO}_*&
    pid1[$i]=`pidof ssh cliente-$i rm /etc/skel/Desktop/${PREFIJO}_*`
    
    scp $TMPDIR/*.desktop cliente-$i:/etc/skel/Desktop&
    pid2[$i]=`pidof scp $TMPDIR/*.desktop cliente-$i:/etc/skel/Desktop`
    
    let i=$i+1
    
    echo $i ; sleep .1
  done

# Esperamos a que se produzcan las copias remotas
  echo "15" ; sleep 1
  echo "30" ; sleep 1
  echo "45" ; sleep 1
  echo "60" ; sleep 1
  echo "75" ; sleep 1

  i=1
  while [ $i -le $NUM_CLIENTES ]
  do
    kill -9 ${pid1[$i]} ${pid2[$i]} ${pid3[$i]}
    let i=$i+1
  done

  echo "100" ; sleep .1

) | $DIALOG --title "$TITULO" --wrap --gauge "Propagando la informacin a los dems ordenadores..." 10 35 

 return 0
}

comprueba_envios() {

  if [ "$NUM_FICHEROS_TRANSMITIR" -eq "0" ]; then
    return 1
  fi

# Comprobamos qu clientes han recibido los ficheros
(  i=1
  while [ $i -le $NUM_CLIENTES ]
  do
    ssh cliente-$i ls /etc/skel/Desktop | xargs -r -n 1 | grep $PREFIJO > /tmp/ficheros_cliente-$i&
    pid1[$i]=`pidof ssh cliente-$i ls /etc/skel/Desktop | xargs -r -n 1 | grep -v $PREFIJO`
    
    let i=$i+1
    
    echo $i ; sleep .1
  done

# Esperamos a que se reciba la informacin de los clientes
  echo "15" ; sleep 1
  echo "30" ; sleep 1
  echo "45" ; sleep 1
  echo "60" ; sleep 1
  echo "75" ; sleep 1

  i=1
  while [ $i -le $NUM_CLIENTES ]
  do
    kill -9 ${pid1[$i]} 
    let i=$i+1
  done

  echo "100" ; sleep .1

) | $DIALOG --title "$TITULO" --wrap --gauge "Comprobando la informacin transmitida..." 10 35 

# Comparamos los ficheros de los clientes operativos con los que haba que
# enviar
   
  ls $TMPDIR | xargs -r -n 1 > /tmp/ficheros_origen
 
  CLIENTES_OK=""
  i=1
  while [ $i -le $NUM_CLIENTES ]
  do
    diff /tmp/ficheros_cliente-$i /tmp/ficheros_origen
    if [ "$?" -eq "0" ]; then # son iguales
      CLIENTES_OK="$CLIENTES_OK\n cliente-$i"
    fi
    let i=$i+1
  done

# Mostramos un resumen con los clientes que han recibido los ficheros
  if [ "$CLIENTES_OK" == "" ]; then # Ningn cliente
    $DIALOG --title "$TITULO" --center --ok-label "Aceptar"  \
            --msgbox " \
No se ha encontrado ningn cliente activo al que mandar las aplicaciones." 0 0 10000
  else
    $DIALOG --title "$TITULO" --center --ok-label "Aceptar" \
            --msgbox " \
Iconos en el escritorio enviados correctamente a los clientes:`echo $CLIENTES_OK`" 0 0 10000
  fi
 
}

muestra_mensaje_inicial
cuenta_ficheros_para_transmitir
prepara_nombres_origen
envia_ficheros_clientes
comprueba_envios

